var LOGFILE_NAME = "myenersave.log";
var l = new Logger(LOGFILE_NAME);

var numValues = 2;
var minResolution = 60;
var verbose = 0;

/**
 * Converts dSID to dSUID
 * @param {string} dSID 24 char long string representing hex value
 * @returns {String} returns a 34 char string representing the dSUID
 */
function dSID2dSUID(dSID) {
    try {
        if (dSID.length === 24) {
            //copy the first 6 bytes (12 characters to the new dsuid
            var dSUID = dSID.slice(0, 12);
            //insert 4 bytes of 0
            dSUID += '00000000';
            //append the remaining 6 bytes (12 chars) of the dsid
            dSUID += dSID.slice(12, 24);
            //finaly add 1 byte of 0
            dSUID += '00';
            return dSUID;
        } else {
            return '';
        }
    } catch (e) {
        l.logln('Exception in backward-compatibility::dSID2dSUID ' + e.name + ' ' + e.description);
        return '';
    }
}

/**
 * Converts property tree dSM entries to dSUID. Stores property tree if anything changed.
 * @returns {undefined}
 */
function convertDSID()
{
    try {
        var iIndex;
        var bConverted = false;
        var meterBaseNode = Property.getNode('dSM');
        var meterArray = meterBaseNode.getChildren();
        for (iIndex = 0; iIndex < meterArray.length; iIndex++) {
            var meterNode = meterArray[iIndex];
            var dSIDNode = meterNode.getChild('dSID');
            if (dSIDNode != null) {
                var dsuid = dsid2dsuid(dSIDNode.getValue());
                var enabled = meterNode.getChild('enabled').getValue();
                Property.setProperty('dSM/' + dsuid + '/dSUID', dsuid);
                Property.setFlag('dSM/' + dsuid + '/dSUID', 'ARCHIVE', true);
                Property.setProperty('dSM/' + dsuid + '/enabled', enabled);
                Property.setFlag('dSM/' + dsuid + '/enabled', 'ARCHIVE', true);         
                meterBaseNode.removeChild(dSIDNode.getValue());
                bConverted = true;
            }
        }
    } catch (e) {
        l.logln('Exception in myenersaver-meter::convertDSID ' + e.name + ' ' + e.description);        
    }
    if (bConverted) {
        Property.store();
    }
}

function addLeadingZeroes(number, count) {
    var result = '' + number;
    while (result.length < count) {
        result = '0' + result;
    }
    return result;
}

function getNeededDSMs()
{
    var oReturnMeters=[],
        Meters=getDSMeters(),
        m;

    for (m in Meters) {
        var oMeter=Meters[m];
        // ignore dsm without metering capabilities
        meteringString = "/apartment/dSMeters/"+ oMeter.dsuid +"/capabilities/metering";
        if (Property.getProperty(meteringString)) {
            if (Property.getNode("dSM/" + oMeter.dsuid + "/dSUID") !== null) {
                if (Property.getProperty("dSM/" + oMeter.dsuid + "/enabled") ) {
                    oReturnMeters.push(oMeter);
                }
            } else {
                if (Property.getProperty("dSM/" + oMeter.dsuid + "/capabilities/metering")) {
                    // also upload values from new dSM
                    oReturnMeters.push(oMeter);
                }
            }
        }
    }
    return oReturnMeters;
}

function sendMeterData() {
    var apikey = Property.getProperty("apiKey");
    if  (!apikey) {
        return;
    }

    var Meters = getNeededDSMs(),
        meterDSUID = [],
        aptValues = 'data=',
        iMeterCounter = 0,
        iValueCounter = 0,
        content = '<upload>\n',
        m;

    for (m in Meters) {
        var oMeter = Meters[m],
            presentString = "/apartment/dSMeters/"+ oMeter.dsuid +"/present";

        if (Property.getProperty(presentString)) {
            var oValues;
            try {
                oValues = Metering.getValues(oMeter.dsuid, "energy",
                    minResolution, "Wh",
                    0, 0,
                    numValues + 2);
            } catch(e) {
                if (verbose >= 1) {
                    l.logln("Cannot request metering values from " + oMeter.dsuid + ":");
                    for (var i in e) {
                        l.logln(i + ': ' + e[i]);
                    }
                }
                oValues = [];
            }
            iMeterCounter++;
            content += '  <sensor id="' + oMeter.dsuid + '" type="10" description="' + oMeter.name + '">\n';

            for (var mIndex = 0; mIndex < oValues.length; mIndex++) {
                var curval = oValues[mIndex],
                    year, mont, day, hour, minute, second, utz, utzVal;

                // value is 0 when meter is not ready yet
                if (curval && curval.value > 0.001) {
                    if (verbose >= 2) {
                        l.logln(" dSM ", oMeter.dsuid, ", value: ", curval.timestamp, " / ", curval.value, "Wh");
                    }
                    iValueCounter++;
                    year = parseInt(curval.timestamp.substring(0, 4), 10);
                    mont = parseInt(curval.timestamp.substring(5, 7), 10);
                    mont -= 1;
                    day = parseInt(curval.timestamp.substring(8, 10), 10);
                    hour = parseInt(curval.timestamp.substring(11, 13), 10);
                    minute = parseInt(curval.timestamp.substring(14, 16), 10);
                    second = parseInt(curval.timestamp.substring(17, 19), 10);

                    utz = new Date(year, mont, day, hour, minute, second, 0);
                    utzVal = (utz.getTime() / 1000) >> 0;

                    content += '   <energy time="' + utzVal + '" reg="'+ curval.value + '"/>\n';
                }
            }
            content += '  </sensor>\n';
        }
    }
    content += '</upload>';
    if (verbose >= 3) {
        l.logln(content);
    }

    if (iMeterCounter === 0) {
        l.logln("No meters selected to upload data from or dSS not ready yet");
        return;
    }

    var sString = "Upload values from " + iMeterCounter + " active dSMeter(s)",
        extraheaders = [];

    extraheaders['Host'] = 'data.myenersave.com:80';
    extraheaders['Connection'] = 'Close';
    extraheaders['Content-Type'] = 'application/xml';
    extraheaders['Content-Length'] = '' + content.length;
    extraheaders['Accept'] = 'text/html';
    extraheaders['User-Agent'] = 'dSS metering';
    extraheaders['Token'] = apikey;

    try {
        var h = new HTTP( { 'debug' : 0 }, extraheaders);
        var code = h.post('http://data.myenersave.com/fetcher/data', '', content, function (returnCode) {
            if (returnCode.status.valueOf() === 200) {
                l.logln(sString + ': OK [200]');
            } else {
                l.logln(sString + ': FAILED! Server replied: "' + returnCode.status + '"');
                if (verbose >= 3) {
                    l.logln(returnCode.body);
                }
            }

        });
    } catch (e) {
        l.logln("Error during upload: ");
        for (var i in e) {
            l.logln(i + ': ' + e[i]);
        }
    }
}

function prepare()
{
    if (raisedEvent.parameter.apikey === null)
        return;

    var prop = new Property('apiKey'),
        meters = getDSMeters(),
        m;

    prop.setValue(raisedEvent.parameter.apikey);
    prop.setFlag('ARCHIVE', true);

    for (m in meters) {
        var oMeter = meters[m];
        // ignore dsm without metering capabilities
        meteringString = "/apartment/dSMeters/"+ oMeter.dsuid +"/capabilities/metering";
        if (Property.getProperty(meteringString)) {
            if (Property.getNode("dSM/" + oMeter.dsuid + "/dSUID") === null) {
                prop = new Property("dSM/" + oMeter.dsuid + "/dSUID");
                prop.setValue(oMeter.dsuid);
                prop.setFlag('ARCHIVE', true);
                prop = new Property("dSM/" + oMeter.dsuid + "/enabled");
                prop.setValue(true);
                prop.setFlag('ARCHIVE', true);
            }
        }
    }

    Property.store();
    sendMeterData();
}

function uploadCleanup()
{
    var id = Property.getProperty('id');
    var evtNode = Property.getNode('/system/EventInterpreter/ScheduledEvents');
    var propNode = evtNode.getChild(id);
    if (propNode) {
        l.logln("Cleanup old schedule " + id);
        propNode.getParent().removeChild(propNode);
    }
    Property.setProperty('id', 0);
}

function uploadScheduler()
{
    uploadCleanup();

    var ecount = Property.getProperty('count');
    Property.setProperty('count', ecount+1);

    var d = new Date();
    var diff = (numValues * minResolution) - d.getSeconds() + 20;

    var uploadEvent = new TimedEvent("myenersave", "+" + diff, { 'actions' : 'sendMeterData' });
    var id = uploadEvent.raise();
    Property.setProperty('id', id);

    if (verbose >= 1) {
        l.logln("Reschedule my id " + id + " in " + diff + "s");
    }
}

function main()
{
    if (raisedEvent.name === "running") {
        Property.load();
        convertDSID();
    }

    else if (raisedEvent.name === "myenersave") {
        if (raisedEvent.parameter.actions) {

            if (verbose >= 1) {
                l.logln('Execute: ' + raisedEvent.name + ' ' + raisedEvent.parameter.actions);
            }

            if (raisedEvent.parameter.actions == 'load') {
                Property.load();
            }
            if (raisedEvent.parameter.actions == 'save') {
                Property.store();
            }
            if (raisedEvent.parameter.actions == 'prepare') {
                prepare();
            }
            if (raisedEvent.parameter.actions == 'ready') {
                var prop = new Property('count');
                prop.setValue(0);
                uploadScheduler();
            }
            if (raisedEvent.parameter.actions == 'sendMeterData') {
                sendMeterData();
                uploadScheduler();
            }
        }
    }
}

main();
